@@ -1,10 +1,10 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
     Grig:  Gtk+ user interface for the Hamradio Control Libraries.
 
-    Copyright (C)  2001-2006  Alexandru Csete.
+    Copyright (C)  2001-2007  Alexandru Csete.
 
-    Authors: Alexandru Csete <csete@users.sourceforge.net>
+    Authors: Alexandru Csete <oz9aec@gmail.com>
 
     Comments, questions and bugreports should be submitted via
     http://sourceforge.net/projects/groundstation/
@@ -26,8 +26,6 @@
     along with this program; if not, visit http://www.fsf.org/
  
  
- 
- 
 */
 /** \file    main.c
  *  \ingroup main
@@ -51,12 +49,15 @@
 #ifdef HAVE_GETOPT_H
 #  include <getopt.h>
 #endif
+#include "compat.h"
 #include "grig-config.h"
 #include "rig-gui.h"
 #include "grig-debug.h"
 #include "rig-gui-message-window.h"
 #include "rig-daemon.h"
 #include "rig-data.h"
+#include "rig-selector.h"
+#include "key-press-handler.h"
 
 
 
@@ -74,14 +75,14 @@ static gchar   *civaddr   = NULL;    /*!< CI-V address for ICOM rig. */
 static gchar   *rigconf   = NULL;    /*!< Configuration parameter. */
 static gint     rigspeed  = 0;       /*!< Optional serial speed. */
 static gboolean listrigs  = FALSE;   /*!< List supported radios and exit. */ 
-gint     debug     = RIG_DEBUG_NONE; /*!< Hamlib debug level. Note: not static since menubar.c needs access. */
+gint debug     = RIG_DEBUG_NONE; /*!< Hamlib debug level. Note: not static since menubar.c needs access. */
 static gint     delay     = 0;       /*!< Command delay. */
 static gboolean nothread  = FALSE;   /*!< Don't use threads, just a regular gtk-timeout. */
 static gboolean pstat     = FALSE;   /*!< Enable power status button. */
 static gboolean ptt       = FALSE;   /*!< Enable PTT button. */
 static gboolean version   = FALSE;   /*!< Show version and exit. */
 static gboolean help      = FALSE;   /*!< Show help and exit. */
-
+//static gchar    *rigcfg   = NULL;    /*!< .radio file name. */
 
 /* group those which take no arg */
 /** \brief Short options. */
@@ -99,7 +100,7 @@ static struct option long_options[] =
 	{"delay",        1, 0, 'D'},
 	{"nothread",     0, 0, 'n'},
 	{"list",         0, 0, 'l'},
-        {"enable-ptt",   0, 0, 'p'},
+	{"enable-ptt",   0, 0, 'p'},
 	{"enable-pwr",   0, 0, 'P'},
 	{"help",         0, 0, 'h'},
 	{"version",      0, 0, 'v'},
@@ -110,7 +111,7 @@ static struct option long_options[] =
 /** \brief Radio info to be used by list-rigs. */
 typedef struct {
 	gint    id;       /*!< Model ID. */
-	gchar  *mfg;      /*!< Manufacurer name (eg. KENWOOD). */
+	gchar  *mfg;      /*!< Manufacturer name (eg. KENWOOD). */
 	gchar  *model;    /*!< Radio model (eg. TS-440). */
 	gchar  *version;  /*!< Driver version (eg. 0.3.2) */
         gint    status;   /*!< Driver status (0..5 use rig_strstatus). */
@@ -161,7 +162,8 @@ main (int argc, char *argv[])
 	   can avoid surprises later on, when exit
 	   is not an option anymore.
 	*/
-	fname = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S, "smeter.png", NULL);
+	//fname = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S, "smeter.png", NULL);
+	fname = pixmap_file_name ("smeter.png");
 	if (!g_file_test (fname, G_FILE_TEST_EXISTS)) {
 
 		g_print ("\n\n");
@@ -169,12 +171,13 @@ main (int argc, char *argv[])
 		g_print (_("This usually means that your installation is incomplete.\n"));
 		g_print (_("Sorry... but I can not continue..."));
 		g_print ("\n\n");
-
+		g_print ("%s\n\n",fname);
 		return 1;
 	}
 
 	g_free (fname);
 
+
 	/* initialize threads; according to glib docs, this call will terminate
 	   the program if threads are not supported... then why doesn''t it work
 	   on FreeBSD?
@@ -183,12 +186,12 @@ main (int argc, char *argv[])
 		g_thread_init (NULL);
 
 
+
 	/* decode command line arguments; this part of the code only sets the
 	   global flags and variables, whereafter we check each variable in
 	   descending priority order. This way it is easy to exit the program
 	   in case of -v -h and such.
 	*/
-	
 	while(1) {
 		int c;
 		int option_index = 0;
@@ -337,23 +340,49 @@ main (int argc, char *argv[])
 	/* initialise debug handler */
 	grig_debug_init (NULL);
 
+	/* check configuration */
+	if (!grig_config_check ()) {
+
+		g_print ("\n\n");
+		g_print (_("Grig configuration check failed!\n"));
+		g_print (_("This usually means that your configuration is broken.\n"));
+		g_print (_("Sorry... but I can not continue..."));
+		g_print ("\n\n");
+		g_print (_("Proposed solutions:\n"));
+		return 1;
+
+	}
+
+	/* At this point, configuration is OK. */
+
+
+    /* 1. prio: .grc file */
+    
+    /* 2. prio: -m or --model */
+    
+    /* 3. prio: run rig-selector */
+    //g_print ("SELECT: %s\n", rig_selector_execute ());
+
 	/* launch rig daemon and pass the relevant
 	   command line options
 	*/
 	if (rig_daemon_start (rignum,
-			      rigfile,
-			      rigspeed,
-			      civaddr,
-			      rigconf,
-			      delay,
-			      nothread,
-			      ptt,
-			      pstat))
+						  rigfile,
+						  rigspeed,
+						  civaddr,
+						  rigconf,
+						  delay,
+						  nothread,
+						  ptt,
+						  pstat))
 	{
 
 		return 1;
 	}
 
+    /* install key press event handler */
+    key_press_handler_init ();
+
 	/* check whether the debug level is something meaningful
 	   (it could be set to something junk by user); if yes, set
 	   debuglevel, otherwise use RIG_DEBUG_WARN.
@@ -370,10 +399,11 @@ main (int argc, char *argv[])
 
 	/* add contents */
 	gtk_container_add (GTK_CONTAINER (grigapp), rig_gui_create ());
-	gtk_widget_show_all (grigapp);
-
+	gtk_widget_show (grigapp);
+    
 	gtk_main ();
 
+
 	return 0;
 }
 
@@ -408,7 +438,8 @@ grig_app_create       (gint rignum)
 	title = g_strdup_printf (_("GRIG: %s %s"), brand, model);
 
 	/* window icon file */
-	icon = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S, "ic910.png", NULL);
+	//icon = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S, "ic910.png", NULL);
+	icon = pixmap_file_name ("ic910.png");
 
 	/* create application */
 	app = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -498,6 +529,9 @@ grig_app_destroy    (GtkWidget *widget,
 	/* set debug level to TRACE */
 	grig_debug_set_level (RIG_DEBUG_TRACE);
 
+    /* remove key press event handler */
+    key_press_handler_close ();
+    
 	/* stop daemons */
 	rig_daemon_stop ();
 
@@ -506,8 +540,8 @@ grig_app_destroy    (GtkWidget *widget,
 	/* stop timeouts */
 
 	/* shut down debug handler */
-      	grig_debug_close ();
-
+    grig_debug_close ();
+    
 	/* exit Gtk+ */
 	gtk_main_quit ();
 }
@@ -599,7 +633,7 @@ grig_show_version   ()
 	g_print (_("Graphical User Interface for the "\
 		   "Hamradio Control Libraries."));
 	g_print ("\n\n");
-	g_print (_("Copyright (C)  2001-2006  Alexandru Csete."));
+	g_print (_("Copyright (C)  2001-2007  Alexandru Csete."));
 	g_print ("\n");
 	g_print (_("This is free software; see the source for "\
 		   "copying conditions. "));
@@ -650,8 +684,8 @@ grig_list_rigs ()
 	g_array_sort (array, grig_list_compare);
 
 	g_print ("\n");
-	g_print ("   ID  Manufacurer      Model                  "\
-		 "Ver.   Status\n");
+	g_print (_("   ID  Manufacturer     Model                  "\
+		 "Ver.   Status\n"));
 	g_print ("-----------------------------------------------"\
 		 "----------------\n");
 
