@@ -1,29 +1,29 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
-    Grig:  Gtk+ user interface for the Hamradio Control Libraries.
+  Grig:  Gtk+ user interface for the Hamradio Control Libraries.
 
-    Copyright (C)  2001-2006  Alexandru Csete.
+  Copyright (C)  2001-2007  Alexandru Csete.
 
-    Authors: Alexandru Csete <csete@users.sourceforge.net>
+  Authors: Alexandru Csete <oz9aec@gmail.com>
 
-    Comments, questions and bugreports should be submitted via
-    http://sourceforge.net/projects/groundstation/
-    More details can be found at the project home page:
+  Comments, questions and bugreports should be submitted via
+  http://sourceforge.net/projects/groundstation/
+  More details can be found at the project home page:
 
-            http://groundstation.sourceforge.net/
+  http://groundstation.sourceforge.net/
  
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
   
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, visit http://www.fsf.org/
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, visit http://www.fsf.org/
  
  
  
@@ -54,12 +54,12 @@
  *
  * The comma need not have the same size as the digits; it may be smaller.
  * When the  pixmap is read, the size of the digits is calclated as follows:
-\code
-        DIGIT_WIDTH  = IMG_WIDTH div 12
-	COMMA_WIDTH  = IMG_WIDTH mod 12 - 1  (first column contains color info)
-	DIGIT_HEIGHT = IMG_HEIGHT
+ \code
+ DIGIT_WIDTH  = IMG_WIDTH div 12
+ COMMA_WIDTH  = IMG_WIDTH mod 12 - 1  (first column contains color info)
+ DIGIT_HEIGHT = IMG_HEIGHT
 
-\endcode
+ \endcode
  * It is therefore quite important that IMG_WIDTH mod 11 > 0 and that the
  * width of the decimal separator is less than 10 pixels.
  *
@@ -82,6 +82,7 @@
 #  include <config.h>
 #endif
 #include <hamlib/rig.h>
+#include "compat.h"
 #include "rig-data.h"
 #include "grig-gtk-workarounds.h"
 #include "rig-gui-lcd.h"
@@ -95,6 +96,7 @@
  */
 typedef enum {
 	EVENT_OBJECT_NONE = -1,      /*!< No object. */
+	EVENT_OBJECT_FREQ_1G,       /*!< 1 GHz frequency digit. */
 	EVENT_OBJECT_FREQ_100M,     /*!< 100 MHz frequency digit. */
 	EVENT_OBJECT_FREQ_10M,      /*!< 10 MHz frequency digit. */
 	EVENT_OBJECT_FREQ_1M,       /*!< 1 MHz frequency digit. */
@@ -130,9 +132,8 @@ static gboolean       rig_gui_lcd_expose_cb        (GtkWidget *, GdkEventExpose
 static gboolean       rig_gui_lcd_handle_event     (GtkWidget *, GdkEvent *, gpointer);
 static event_object_t rig_gui_lcd_get_event_object (GdkEvent *event);
 static void           rig_gui_lcd_calc_dim         (void);
-static void           rig_gui_lcd_set_freq_digits  (freq_t freq);
-static void           rig_gui_lcd_set_rit_digits   (shortfreq_t freq);
 static void           rig_gui_lcd_draw_text        (void);
+static void	      rig_gui_lcd_draw_digit	   (gint position, char digit);
 
 static gint           rig_gui_lcd_timeout_exec     (gpointer);
 static gint           rig_gui_lcd_timeout_stop     (gpointer);
@@ -150,12 +151,12 @@ static void           rig_gui_lcd_update_vfo       (void);
 GtkWidget *
 rig_gui_lcd_create ()
 {
-	GtkWidget *vbox;
 	guint      timerid;
 	guint      i;
 
 	/* init data */
 	lcd.exposed = FALSE;
+	lcd.manual = FALSE;
 
 	/* load digit pixmaps from file */
 	rig_gui_lcd_load_digits (NULL);
@@ -164,7 +165,7 @@ rig_gui_lcd_create ()
 	rig_gui_lcd_calc_dim ();
 
 	/* clear freqs buffers */
-	for (i=0; i<9; i++) {
+	for (i=0; i<10; i++) {
 		lcd.freqs1[i] = 'X';
 	}
 
@@ -173,6 +174,10 @@ rig_gui_lcd_create ()
 		lcd.xits[i] = 'X';
 	}
 
+	g_signal_new("freq-changed", GTK_TYPE_WIDGET,
+		G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION, 0, NULL,
+		NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+
 	/* create canvas */
 	lcd.canvas = gtk_drawing_area_new ();
 	gtk_widget_set_size_request (lcd.canvas, lcd.width, lcd.height);
@@ -181,10 +186,10 @@ rig_gui_lcd_create ()
 	   contents.
 	*/
 	g_signal_connect (G_OBJECT (lcd.canvas), "expose_event",  
-			  G_CALLBACK (rig_gui_lcd_expose_cb), NULL);	
+                      G_CALLBACK (rig_gui_lcd_expose_cb), NULL);	
 
 	/* connect mouse events but only if rig has set_freq;
-	   THIS IS A BUG SINCE WE DON'T DISTINGUISH BETWEEN SET_FREQ
+	   XXX THIS IS A BUG SINCE WE DON'T DISTINGUISH BETWEEN SET_FREQ
 	   AND SET_RIT/SET_XIT.
 	*/
 #ifndef DISABLE_HW
@@ -192,20 +197,11 @@ rig_gui_lcd_create ()
 #endif
 		gtk_widget_add_events (lcd.canvas, GDK_BUTTON_PRESS_MASK);
 		g_signal_connect (G_OBJECT (lcd.canvas), "event",
-				  G_CALLBACK (rig_gui_lcd_handle_event), NULL);
+                          G_CALLBACK (rig_gui_lcd_handle_event), NULL);
 #ifndef DISABLE_HW
 	}
 #endif
 
-	/* control widgets */
-
-
-	/* pack LCD and controls into a vertical box */
-	vbox = gtk_vbox_new (FALSE, 0);
-
-	gtk_box_pack_start (GTK_BOX (vbox), lcd.canvas, FALSE, FALSE, 5);
-//	gtk_box_pack_start (GTK_BOX (vbox), hbox,  FALSE, FALSE, 0);
-
 	/* start readback timer but only if service is available 
 	   or we are in DISABLE_HW mode
 	*/
@@ -213,17 +209,19 @@ rig_gui_lcd_create ()
 	if (rig_data_has_get_freq1 ()) {
 #endif
 		timerid = g_timeout_add (RIG_GUI_LCD_DEF_TVAL,
-					 rig_gui_lcd_timeout_exec,
-					 NULL);
+                                 rig_gui_lcd_timeout_exec,
+                                 NULL);
 
 		/* register timer_stop function at exit */
 		gtk_quit_add (gtk_main_level (), rig_gui_lcd_timeout_stop,
-			      GUINT_TO_POINTER (timerid));
+                      GUINT_TO_POINTER (timerid));
 #ifndef DISABLE_HW
 	}
 #endif
 
-	return vbox;
+    gtk_widget_show_all (lcd.canvas);
+    
+	return lcd.canvas;
 }
 
 
@@ -236,10 +234,10 @@ rig_gui_lcd_create ()
  * obtains the drawing area bg/fg colors based on the loaded
  * digits. Because the first column in the pixmap contains the background and
  * foreground colors, the dimensions of each digits is calculated as follows:
-\code
-               W = (pixmap_width - 1) / 12
-               H = pixmap_height
-\endcode
+ \code
+ W = (pixmap_width - 1) / 12
+ H = pixmap_height
+ \endcode
  * The parameter name is used to specify the files from which the digits should
  * be loaded. If name is NULL, the default pixmaps will be read. Otherwise grig
  * will look for $PACKAGE_PIXMAPS_DIR/name_normal.png and
@@ -256,17 +254,22 @@ rig_gui_lcd_load_digits (const gchar *name)
 	guint dw,dh,cw;      /* width and height of a digit and width of comma */
 	gint bps,rs;         /* bits pr.ample and rowstride */
 	gchar *fname;
+	gchar *tmp;
 	guchar *pixels;
 
 	
 	/* normal digits */
 	if (name == NULL) {
-		fname = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S,
-				     "digits_normal.png", NULL);
+		//fname = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S,
+		//		     "digits_normal.png", NULL);
+		fname = pixmap_file_name ("digits_normal.png");
 	}
 	else {
-		fname = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S,
-				     name, "_normal.png", NULL);
+		//fname = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S,
+		//		     name, "_normal.png", NULL);
+		tmp = g_strconcat (name, "_normal.png", NULL);
+		fname = pixmap_file_name (tmp);
+		g_free (tmp);
 	}
 
 	/* load pixmap */
@@ -310,12 +313,16 @@ rig_gui_lcd_load_digits (const gchar *name)
 
 	/* small digits */
 	if (name == NULL) {
-		fname = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S,
-				     "digits_small.png", NULL);
+		//fname = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S,
+		//		     "digits_small.png", NULL);
+		fname = pixmap_file_name ("digits_small.png");
 	}
 	else {
-		fname = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S,
-				     name, "_small.png", NULL);
+		//fname = g_strconcat (PACKAGE_PIXMAPS_DIR, G_DIR_SEPARATOR_S,
+		//		     name, "_small.png", NULL);
+		tmp = g_strconcat (name, "_small.png", NULL);
+		fname = pixmap_file_name (tmp);
+		g_free (tmp);
 	}
 
 	/* load pixmap */
@@ -351,9 +358,9 @@ rig_gui_lcd_load_digits (const gchar *name)
  *  \param event  The event.
  *  \param data   User data; always NULL.
  * 
- * This function is called when the rawing area widget is finalized
- * and exposed. Itis used to finish the initialization of those
- * parameters, which need attributes rom visible widgets.
+ * This function is called when the drawing area widget is finalized
+ * and exposed. It is used to finish the initialization of those
+ * parameters, which need attributes from visible widgets.
  *
  * \bug canvas height is hadcoded according to smeter height.
  *
@@ -361,8 +368,8 @@ rig_gui_lcd_load_digits (const gchar *name)
  */ 
 static gboolean
 rig_gui_lcd_expose_cb   (GtkWidget      *widget,
-			 GdkEventExpose *event,
-			 gpointer        data)
+                         GdkEventExpose *event,
+                         gpointer        data)
 {
 
 	guint i;
@@ -372,29 +379,29 @@ rig_gui_lcd_expose_cb   (GtkWidget      *widget,
 	gdk_gc_set_rgb_fg_color (lcd.gc1, &lcd.fg);
 	gdk_gc_set_rgb_bg_color (lcd.gc1, &lcd.bg);
 	gdk_gc_set_line_attributes (lcd.gc1, 1,
-				    GDK_LINE_SOLID,
-				    GDK_CAP_ROUND,
-				    GDK_JOIN_ROUND);
+                                GDK_LINE_SOLID,
+                                GDK_CAP_ROUND,
+                                GDK_JOIN_ROUND);
 
 	lcd.gc2 = gdk_gc_new (GDK_DRAWABLE (widget->window));
 	gdk_gc_set_rgb_fg_color (lcd.gc2, &lcd.bg);
 	gdk_gc_set_rgb_bg_color (lcd.gc2, &lcd.fg);
 	gdk_gc_set_line_attributes (lcd.gc2, 1,
-				    GDK_LINE_SOLID,
-				    GDK_CAP_ROUND,
-				    GDK_JOIN_ROUND);
+                                GDK_LINE_SOLID,
+                                GDK_CAP_ROUND,
+                                GDK_JOIN_ROUND);
 
 				    
 	/* draw border around the meter */
 	gdk_draw_rectangle (GDK_DRAWABLE (widget->window), lcd.gc2,
-			    TRUE, 0, 0, lcd.width, lcd.height);
+                        TRUE, 0, 0, lcd.width, lcd.height);
 
 	gdk_draw_rectangle (GDK_DRAWABLE (widget->window), lcd.gc1,
-			    FALSE, 0, 0, lcd.width-1, lcd.height-1);
+                        FALSE, 0, 0, lcd.width-1, lcd.height-1);
 
 
 	/* force digit update by clearing internal string buffer */
-	for (i=0; i<9; i++) {
+	for (i=0; i<10; i++) {
 		lcd.freqs1[i] = 'X';
 	}
 	rig_gui_lcd_set_freq_digits (lcd.freq1);
@@ -405,30 +412,30 @@ rig_gui_lcd_expose_cb   (GtkWidget      *widget,
 	}
 	rig_gui_lcd_set_rit_digits (lcd.rit);
 
-	/* large dots */
-/* 	gdk_draw_pixbuf (GDK_DRAWABLE (widget->window), NULL, digits_normal[12], */
-/* 			 0, 0, lcd.others[0].x, lcd.others[0].y, -1, -1, */
-/* 			 GDK_RGB_DITHER_NONE, 0, 0); */
-
+	/* large dot */
 	gdk_draw_pixbuf (GDK_DRAWABLE (widget->window), NULL, digits_normal[12],
-			 0, 0, lcd.others[1].x, lcd.others[1].y, -1, -1,
-			 GDK_RGB_DITHER_NONE, 0, 0);
+	                 0, 0, lcd.dots[0].x, lcd.dots[0].y, -1, -1,
+	                 GDK_RGB_DITHER_NONE, 0, 0);
 
 	/* small dot */
 	gdk_draw_pixbuf (GDK_DRAWABLE (widget->window), NULL, digits_small[12],
-			 0, 0, lcd.others[2].x, lcd.others[2].y, -1, -1,
-			 GDK_RGB_DITHER_NONE, 0, 0);
+	                 0, 0, lcd.dots[1].x, lcd.dots[1].y, -1, -1,
+	                 GDK_RGB_DITHER_NONE, 0, 0);
 
 
 
 
 	/* initialize offscreen buffer */
 	buffer = gdk_pixmap_new (GDK_DRAWABLE (lcd.canvas->window),
-				 lcd.width, lcd.height, -1);
+                             lcd.width, lcd.height, -1);
 
 	/* draw text */
 	rig_gui_lcd_draw_text ();
 
+	/* force VFO update by clearing internal vfo state */
+	lcd.vfo = RIG_VFO_NONE;
+	rig_gui_lcd_update_vfo ();
+
 	/* indicate that widget is ready to 
 	   be used
 	*/
@@ -463,8 +470,8 @@ rig_gui_lcd_expose_cb   (GtkWidget      *widget,
  */
 static gboolean
 rig_gui_lcd_handle_event     (GtkWidget *widget,
-			      GdkEvent  *event,
-			      gpointer   data)
+                              GdkEvent  *event,
+                              gpointer   data)
 {
 	event_object_t object;     /* event object */
 	freq_t         deltaf;     /* frequency change */
@@ -477,7 +484,7 @@ rig_gui_lcd_handle_event     (GtkWidget *widget,
 	/* in case of expose-event call the expose event handler */
 	switch (event->type) {
 
-		/* drawing area hss been exposed */
+		/* drawing area has been exposed */
 	case GDK_EXPOSE:
 		return rig_gui_lcd_expose_cb (widget, (GdkEventExpose *) event, data);
 		break;
@@ -495,13 +502,13 @@ rig_gui_lcd_handle_event     (GtkWidget *widget,
 		else if (object > EVENT_OBJECT_FREQ_1) {
 
 			/* RIT/XIT event;
-			   object is in the range 9..11
+			   object is in the range 10..12
 			*/
-			power = 12 - object;
+			power = 13 - object;
 			deltar = pow (10, power);
 
 			if (deltar < rig_data_get_ritstep ()) {
-//				return TRUE;
+                //				return TRUE;
 				deltar = rig_data_get_ritstep ();
 			}
 
@@ -537,7 +544,7 @@ rig_gui_lcd_handle_event     (GtkWidget *widget,
 				   not 0-terminated; it's just a byte array!
 				*/
 				str = g_strdup_printf ("%5d", lcd.rit);
-				str[object-8] = '0';
+				str[object-9] = '0';
 
 				newrit = (freq_t) g_strtod (str, NULL);
 
@@ -579,14 +586,14 @@ rig_gui_lcd_handle_event     (GtkWidget *widget,
 		}  /* else if object > EVENT_OBJECT_FREQ_1 */
 
 		else {  /* frequency event;
-			   object is in the range 0..8 with
-			   0 corresponding to 100 MHz
-			*/
-			power = 8 - object;
+                   object is in the range 0..9 with
+                   0 corresponding to 1 GHz
+                */
+			power = 9 - object;
 			deltaf = pow (10, power);
 
 			if (deltaf < rig_data_get_fstep ()) {
-//				return TRUE;
+                //				return TRUE;
 				deltaf = rig_data_get_fstep ();
 			}
 
@@ -617,7 +624,7 @@ rig_gui_lcd_handle_event     (GtkWidget *widget,
 				   WARNING: don't use lcd.freqs1 because it is
 				   not 0-terminated; it's just a byte array!
 				*/
-				str = g_strdup_printf ("%9.0f", lcd.freq1);
+				str = g_strdup_printf ("%10.0f", lcd.freq1);
 				str[object] = '0';
 
 				newfreq = (freq_t) g_strtod (str, NULL);
@@ -671,13 +678,13 @@ rig_gui_lcd_handle_event     (GtkWidget *widget,
 		else if (object > EVENT_OBJECT_FREQ_1) {
 
 			/* RIT/XIT event;
-			   object is in the range 9..11
+			   object is in the range 10..12
 			*/
-			power = 12 - object;
+			power = 13 - object;
 			deltar = pow (10, power);
 
 			if (deltar < rig_data_get_ritstep ()) {
-//				return TRUE;
+                //				return TRUE;
 				deltar = rig_data_get_ritstep ();
 			}
 
@@ -730,14 +737,14 @@ rig_gui_lcd_handle_event     (GtkWidget *widget,
 		}
 
 		else {	/* frequency event
-			   object is in the range 0..8 with
-			   0 corresponding to 100 MHz
-			*/
-			power = 8 - object;
+                   object is in the range 0..9 with
+                   0 corresponding to 1 GHz
+                */
+			power = 9 - object;
 			deltaf = pow (10, power);
 
 			if (deltaf < rig_data_get_fstep ()) {
-//				return TRUE;
+                //				return TRUE;
 				deltaf = rig_data_get_fstep ();
 			}
 
@@ -810,7 +817,7 @@ static event_object_t
 rig_gui_lcd_get_event_object (GdkEvent *event)
 {
 	guint x,y;    /* coordinates */
-	guint i;
+    gint i;
 
 	x = (guint) ((GdkEventButton*)event)->x;
 	y = (guint) ((GdkEventButton*)event)->y;
@@ -823,9 +830,9 @@ rig_gui_lcd_get_event_object (GdkEvent *event)
 
 	/* check x */
 	if ((x < lcd.digits[0].x)                                     || 
-	    (x > lcd.digits[11].x+lcd.dsw)                            ||
-	    ((x > lcd.digits[8].x+lcd.dsw) && (x < lcd.digits[9].x))  ||
-	    ((x > lcd.digits[9].x+lcd.dsw) && (x < lcd.digits[10].x)))   {
+	    (x > lcd.digits[12].x+lcd.dsw)                            ||
+	    ((x > lcd.digits[9].x+lcd.dsw) && (x < lcd.digits[10].x))  ||
+	    ((x > lcd.digits[10].x+lcd.dsw) && (x < lcd.digits[11].x)))   {
 		
 		return EVENT_OBJECT_NONE;
 	}
@@ -836,7 +843,7 @@ rig_gui_lcd_get_event_object (GdkEvent *event)
 		   are changed more often.
 		*/
 		/* small digits */
-		for (i=11; i>5; i--) {
+		for (i=12; i>6; i--) {
 			
 			if ((x > lcd.digits[i].x) && (x < lcd.digits[i].x+lcd.dsw)) {
 				return i;
@@ -845,7 +852,7 @@ rig_gui_lcd_get_event_object (GdkEvent *event)
 		}
 		
 		/* then try large digits */
-		for (i=5; i>=0; i--) {
+		for (i=6; i>=0; i--) {
 			
 			if ((x > lcd.digits[i].x) && (x < lcd.digits[i].x+lcd.dlw)) {
 				return i;
@@ -891,7 +898,7 @@ rig_gui_lcd_calc_dim    ()
 	lcd.csw = gdk_pixbuf_get_width  (digits_small[12]);
 
 	/* calculate drawing area dimensions */
-	lcd.width = 6*lcd.dlw + 2*lcd.clw + 8*lcd.dsw + lcd.csw + 2*LCD_MARGIN;
+	lcd.width = 7*lcd.dlw + 3*lcd.clw + 8*lcd.dsw + lcd.csw + 2*LCD_MARGIN;
 	lcd.height = 80;
 
 	/* calculate screen position for each digit; this will ease the
@@ -899,31 +906,30 @@ rig_gui_lcd_calc_dim    ()
 	*/
 	/* VFO digits */
 	lcd.digits[0].x = LCD_MARGIN / 2;
-	lcd.digits[1].x = lcd.digits[0].x + lcd.dlw;
+	lcd.digits[1].x = lcd.digits[0].x + lcd.clw + lcd.dlw;
 	lcd.digits[2].x = lcd.digits[1].x + lcd.dlw;
-	lcd.digits[3].x = lcd.digits[2].x + lcd.clw + lcd.dlw;   /**/
-	lcd.digits[4].x = lcd.digits[3].x + lcd.dlw;
+	lcd.digits[3].x = lcd.digits[2].x + lcd.dlw;
+	lcd.digits[4].x = lcd.digits[3].x + lcd.clw + lcd.dlw;   /**/
 	lcd.digits[5].x = lcd.digits[4].x + lcd.dlw;
-	lcd.digits[6].x = lcd.digits[5].x + lcd.clw + lcd.dlw;   /**/
-	lcd.digits[7].x = lcd.digits[6].x + lcd.dsw;
+	lcd.digits[6].x = lcd.digits[5].x + lcd.dlw;
+	lcd.digits[7].x = lcd.digits[6].x + lcd.clw + lcd.dlw;   /**/
 	lcd.digits[8].x = lcd.digits[7].x + lcd.dsw;
-	lcd.digits[9].x = lcd.digits[8].x + 4*lcd.dsw; /**/
-	lcd.digits[10].x = lcd.digits[9].x + lcd.csw + lcd.dsw;   /**/
-	lcd.digits[11].x = lcd.digits[10].x + lcd.dsw;
+	lcd.digits[9].x = lcd.digits[8].x + lcd.dsw;
+	lcd.digits[10].x = lcd.digits[9].x + 4*lcd.dsw; /**/
+	lcd.digits[11].x = lcd.digits[10].x + lcd.csw + lcd.dsw;   /**/
+	lcd.digits[12].x = lcd.digits[11].x + lcd.dsw;
 	
-	for (i=0; i<6; i++)
+	for (i=0; i<7; i++)
 		lcd.digits[i].y = (lcd.height - lcd.dlh)/2;
 
-	for (i=6; i<13; i++)
+	for (i=7; i<13; i++)
 		lcd.digits[i].y = lcd.digits[1].y + (lcd.dlh-lcd.dsh)-1;
 
-	lcd.others[0].x = lcd.digits[2].x + lcd.dlw;
-	lcd.others[1].x = lcd.digits[5].x + lcd.dlw;
-	lcd.others[2].x = lcd.digits[9].x + lcd.dsw;
+	lcd.dots[0].x = lcd.digits[6].x + lcd.dlw;
+	lcd.dots[1].x = lcd.digits[10].x + lcd.dsw;
 
-	lcd.others[0].y = (lcd.height - lcd.dlh)/2;
-	lcd.others[1].y = (lcd.height - lcd.dlh)/2;
-	lcd.others[2].y = lcd.digits[1].y + (lcd.dlh-lcd.dsh)-1;
+	lcd.dots[0].y = (lcd.height - lcd.dlh)/2;
+	lcd.dots[1].y = lcd.digits[1].y + (lcd.dlh-lcd.dsh)-1;
 
 
 }
@@ -936,7 +942,7 @@ rig_gui_lcd_calc_dim    ()
  * The frequency is received in hamlib format (float) and converted to a
  * string with 9 digits. If the frequency is less than 1 GHz the obtained
  * resolution will be 1 Hz, while for frequenciesabove 1 GHz the resolution
- * will be 1 kHz.
+ * will be 1 kHz
  *
  * \note The function is optimized in the sense that before drawing of each digit
  * it is checked whether the new digit is different from the one already beeing
@@ -946,124 +952,203 @@ rig_gui_lcd_calc_dim    ()
  *
  * \sa rig_gui_lcd_set_rit_digits
  */
-static void
+void
 rig_gui_lcd_set_freq_digits  (freq_t freq)
 {
 	gchar *str;   /* frequency as a string */
 	guint  i;     /* iterator */
-
+	gboolean changed = FALSE;
 	
 	/* is drawing area ready? */
 	if (!lcd.exposed || (freq < rig_data_get_fmin ()))
 		return;
 
+	if (lcd.manual)
+		return;
+
 	/* saturate frequency */
-	if (freq >= GHz(1))
-		freq = MHz(999.999999);
+	if (freq >= GHz(10))
+		freq = MHz(9999.999999);
 
 	/* store the new frequency for later use */
 	lcd.freq1 = freq;
 
 	/* convert frequency to string */
-	str = g_strdup_printf ("%9.0f", freq);
+	str = g_strdup_printf ("%10.0f", freq);
 
 	/* for each digit check whether the new digit is different from the one
 	   already beeing displayed; if yes, draw the new digit, otherwise do
 	   nothing.
 	*/
-	for (i=0; i<9; i++) {
+
+	for (i=0; i<10; i++) {
 
 		if (str[i] != lcd.freqs1[i]) {
 
+			changed = TRUE;
+
 			lcd.freqs1[i] = str[i];
 
-			switch (str[i]) {
+			rig_gui_lcd_draw_digit(i, str[i]);
+		}
+	}
 
-			case '0':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 (i < 6) ? digits_normal[0] : digits_small[0],
-						 0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+	g_free(str);
 
-			case '1':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 (i < 6) ? digits_normal[1] : digits_small[1],
-						 0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+	if (changed)
+		g_signal_emit_by_name(lcd.canvas, "freq-changed");
+}
 
-			case '2':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 (i < 6) ? digits_normal[2] : digits_small[2],
-						 0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+static void
+rig_gui_lcd_draw_digit(gint position, char digit)
+{
+	gint ipixmap; /* index in pixmap */
 
-			case '3':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 (i < 6) ? digits_normal[3] : digits_small[3],
-						 0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+	switch (digit) {
 
-			case '4':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 (i < 6) ? digits_normal[4] : digits_small[4],
-						 0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+	case '0':
+		ipixmap = 0;
+		break;
 
-			case '5':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 (i < 6) ? digits_normal[5] : digits_small[5],
-						 0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+	case '1':
+		ipixmap = 1;
+		break;
 
-			case '6':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 (i < 6) ? digits_normal[6] : digits_small[6],
-						 0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+	case '2':
+		ipixmap = 2;
+		break;
 
-			case '7':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 (i < 6) ? digits_normal[7] : digits_small[7],
-						 0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+	case '3':
+		ipixmap = 3;
+		break;
 
-			case '8':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 (i < 6) ? digits_normal[8] : digits_small[8],
-						 0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+	case '4':
+		ipixmap = 4;
+		break;
 
-			case '9':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 (i < 6) ? digits_normal[9] : digits_small[9],
-						 0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+	case '5':
+		ipixmap = 5;
+		break;
 
-			case ' ':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 (i < 6) ? digits_normal[10] : digits_small[10],
-						 0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+	case '6':
+		ipixmap = 6;
+		break;
 
+	case '7':
+		ipixmap = 7;
+		break;
 
-			default: /* critical error */
-				break;
-			}
+	case '8':
+		ipixmap = 8;
+		break;
 
+	case '9':
+		ipixmap = 9;
+		break;
+
+	case ' ':
+		ipixmap = 10;
+		break;
+
+	case '-':
+		ipixmap = 11;
+		break;
+
+	default: /* critical error */
+		return;
+	}
+	gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
+	                 (position < 7) ? digits_normal[ipixmap] : digits_small[ipixmap],
+	                 0, 0, lcd.digits[position].x, lcd.digits[position].y, -1, -1,
+	                 GDK_RGB_DITHER_NONE, 0, 0);
+}
+
+void
+rig_gui_lcd_set_next_digit(char n)
+{
+	/* is drawing area ready? */
+	if (!lcd.exposed)
+		return;
+
+	if (!lcd.manual)
+		return;
+
+	lcd.freqm += pow(10, 9 - lcd.digit) * (n - '0');
+
+	rig_gui_lcd_draw_digit(lcd.digit, n);
+
+	/* increment for next digit */
+	lcd.digit++;
+
+	/* check if this is the last digit and set the freq */
+	if (lcd.digit == 10) {
+		rig_data_set_freq(1, lcd.freqm);
+		rig_gui_lcd_set_freq_digits(lcd.freqm);
+
+		lcd.manual = FALSE;
+	}
+}
+
+void
+rig_gui_lcd_begin_manual_entry  (void)
+{
+	guint i;
+
+	/* is drawing area ready? */
+	if (!lcd.exposed)
+		return;
+
+	/* if we already were in manual eypad mode,
+	 * pad the frequency with zeros
+	 */
+	if (lcd.manual) {
+
+		if (lcd.digit == 0) {
+			/* revert to previous freq if no digit has been entered */
+			rig_gui_lcd_clear_manual_entry();
+			return;
 		}
 
+		for (i = lcd.digit; i < 10; i++) {
+			rig_gui_lcd_set_next_digit('0');
+		}
+
+		return;
 	}
+
+	lcd.manual = TRUE;
+	lcd.digit = 0;
+	lcd.freqm = 0;
+
+	for (i = 0; i < 10; i++) {
+
+		gdk_draw_pixbuf (GDK_DRAWABLE(lcd.canvas->window), NULL,
+			(i < 7) ? digits_normal[11] : digits_small[11],
+			0, 0, lcd.digits[i].x, lcd.digits[i].y, -1, -1,
+			GDK_RGB_DITHER_NONE, 0, 0);
+	}
+}
+
+void
+rig_gui_lcd_clear_manual_entry  (void)
+{
+	gint i;
+
+	/* is drawing area ready? */
+	if (!lcd.exposed)
+		return;
+
+	if (!lcd.manual)
+		return;
+
+	lcd.manual = FALSE;
+
+	/* force digit update by clearing internal string buffer */
+	for (i = 0; i < 10; i++) {
+		lcd.freqs1[i] = 'X';
+	}
+
+	rig_gui_lcd_set_freq_digits(lcd.freq1);
 }
 
 
@@ -1082,7 +1167,7 @@ rig_gui_lcd_set_freq_digits  (freq_t freq)
  *
  * \sa rig_gui_lcd_set_freq_digits
  */
-static void
+void
 rig_gui_lcd_set_rit_digits   (shortfreq_t freq)
 {
 	gchar *str;
@@ -1112,27 +1197,27 @@ rig_gui_lcd_set_rit_digits   (shortfreq_t freq)
 	*/
 	if (str[0] != lcd.rits[0]) {
 
-			lcd.rits[0] = str[0];
+        lcd.rits[0] = str[0];
 
-			switch (str[0]) {
+        switch (str[0]) {
 
-			case ' ':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 digits_small[10], 0, 0,
-						 lcd.digits[9].x - lcd.dsw, lcd.digits[9].y,
-						 -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+        case ' ':
+            gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
+                             digits_small[10], 0, 0,
+                             lcd.digits[10].x - lcd.dsw, lcd.digits[10].y,
+                             -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
+            break;
 
-			case '-':
-				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
-						 digits_small[11], 0, 0,
-						 lcd.digits[9].x - lcd.dsw, lcd.digits[9].y,
-						 -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
-				break;
+        case '-':
+            gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL,
+                             digits_small[11], 0, 0,
+                             lcd.digits[10].x - lcd.dsw, lcd.digits[10].y,
+                             -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
+            break;
 
-			default: /* critical internal error */
-				break;
-			}
+        default: /* critical internal error */
+            break;
+        }
 	}
  
 	/* for each digit check whether the new digit is different from the one
@@ -1150,62 +1235,62 @@ rig_gui_lcd_set_rit_digits   (shortfreq_t freq)
 			case '0':
 			case ' ':
 				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL, digits_small[0],
-						 0, 0, lcd.digits[i+8].x, lcd.digits[i+8].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
+                                 0, 0, lcd.digits[i+9].x, lcd.digits[i+9].y, -1, -1,
+                                 GDK_RGB_DITHER_NONE, 0, 0);
 				break;
 
 			case '1':
 				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL, digits_small[1],
-						 0, 0, lcd.digits[i+8].x, lcd.digits[i+8].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
+                                 0, 0, lcd.digits[i+9].x, lcd.digits[i+9].y, -1, -1,
+                                 GDK_RGB_DITHER_NONE, 0, 0);
 				break;
 
 			case '2':
 				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL, digits_small[2],
-						 0, 0, lcd.digits[i+8].x, lcd.digits[i+8].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
+                                 0, 0, lcd.digits[i+9].x, lcd.digits[i+9].y, -1, -1,
+                                 GDK_RGB_DITHER_NONE, 0, 0);
 				break;
 
 			case '3':
 				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL, digits_small[3],
-						 0, 0, lcd.digits[i+8].x, lcd.digits[i+8].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
+                                 0, 0, lcd.digits[i+9].x, lcd.digits[i+9].y, -1, -1,
+                                 GDK_RGB_DITHER_NONE, 0, 0);
 				break;
 
 			case '4':
 				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL, digits_small[4],
-						 0, 0, lcd.digits[i+8].x, lcd.digits[i+8].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
+                                 0, 0, lcd.digits[i+9].x, lcd.digits[i+9].y, -1, -1,
+                                 GDK_RGB_DITHER_NONE, 0, 0);
 				break;
 
 			case '5':
 				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL, digits_small[5],
-						 0, 0, lcd.digits[i+8].x, lcd.digits[i+8].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
+                                 0, 0, lcd.digits[i+9].x, lcd.digits[i+9].y, -1, -1,
+                                 GDK_RGB_DITHER_NONE, 0, 0);
 				break;
 
 			case '6':
 				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL, digits_small[6],
-						 0, 0, lcd.digits[i+8].x, lcd.digits[i+8].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
+                                 0, 0, lcd.digits[i+9].x, lcd.digits[i+9].y, -1, -1,
+                                 GDK_RGB_DITHER_NONE, 0, 0);
 				break;
 
 			case '7':
 				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL, digits_small[7],
-						 0, 0, lcd.digits[i+8].x, lcd.digits[i+8].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
+                                 0, 0, lcd.digits[i+9].x, lcd.digits[i+9].y, -1, -1,
+                                 GDK_RGB_DITHER_NONE, 0, 0);
 				break;
 
 			case '8':
 				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL, digits_small[8],
-						 0, 0, lcd.digits[i+8].x, lcd.digits[i+8].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
+                                 0, 0, lcd.digits[i+9].x, lcd.digits[i+9].y, -1, -1,
+                                 GDK_RGB_DITHER_NONE, 0, 0);
 				break;
 
 			case '9':
 				gdk_draw_pixbuf (GDK_DRAWABLE (lcd.canvas->window), NULL, digits_small[9],
-						 0, 0, lcd.digits[i+8].x, lcd.digits[i+8].y, -1, -1,
-						 GDK_RGB_DITHER_NONE, 0, 0);
+                                 0, 0, lcd.digits[i+9].x, lcd.digits[i+9].y, -1, -1,
+                                 GDK_RGB_DITHER_NONE, 0, 0);
 				break;
 
 			default: /* critical internal error */
@@ -1318,17 +1403,17 @@ rig_gui_lcd_draw_text        ()
 
 	/* draw text; frequency */
 	gdk_draw_layout (lcd.canvas->window,
-			 lcd.gc1,
-			 lcd.digits[8].x + lcd.dsw + 5,
-			 lcd.digits[8].y + lcd.dsh - h,
-			 layout);
+                     lcd.gc1,
+                     lcd.digits[9].x + lcd.dsw + 5,
+                     lcd.digits[9].y + lcd.dsh - h,
+                     layout);
 
 	/* draw text; rit */
 	gdk_draw_layout (lcd.canvas->window,
-			 lcd.gc1,
-			 lcd.digits[11].x + lcd.dsw + 5,
-			 lcd.digits[11].y + lcd.dsh - h,
-			 layout);
+                     lcd.gc1,
+                     lcd.digits[12].x + lcd.dsw + 5,
+                     lcd.digits[12].y + lcd.dsh - h,
+                     layout);
 
 	rig_gui_lcd_update_vfo ();
 
@@ -1343,10 +1428,10 @@ rig_gui_lcd_draw_text        ()
 
 	/* draw text; RIT */
 	gdk_draw_layout (lcd.canvas->window,
-			 lcd.gc1,
-			 lcd.digits[10].x,
-			 lcd.digits[0].y - h,
-			 layout);
+                     lcd.gc1,
+                     lcd.digits[11].x,
+                     lcd.digits[0].y - h,
+                     layout);
 
 
 	/* free PangoLayout */
@@ -1403,6 +1488,10 @@ rig_gui_lcd_update_vfo ()
 		pango_layout_set_text (layout, _("SUB VFO"), -1);
 		break;
 
+	case RIG_VFO_MEM:
+		pango_layout_set_text (layout, _("MEM"), -1);
+		break;
+
 	default:
 		pango_layout_set_text (layout, _("VFO ?"), -1);
 		break;
@@ -1417,20 +1506,20 @@ rig_gui_lcd_update_vfo ()
 
 	/* clear the area */
 	gdk_draw_rectangle (GDK_DRAWABLE (lcd.canvas->window),
-			    lcd.gc2,
-			    TRUE,
-			    lcd.digits[4].x,
-			    lcd.digits[0].y - h,
-			    2*w,
-			    h);
+                        lcd.gc2,
+                        TRUE,
+                        lcd.digits[5].x,
+                        lcd.digits[0].y - h,
+                        2*w,
+                        h);
 
 
 	/* draw text */
 	gdk_draw_layout (lcd.canvas->window,
-			 lcd.gc1,
-			 lcd.digits[4].x,
-			 lcd.digits[0].y - h,
-			 layout);
+                     lcd.gc1,
+                     lcd.digits[5].x,
+                     lcd.digits[0].y - h,
+                     layout);
 
 	/* free PangoLayout */
 	g_object_unref (G_OBJECT (layout));
