@@ -1,10 +1,10 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
     Grig:  Gtk+ user interface for the Hamradio Control Libraries.
 
-    Copyright (C)  2001-2006  Alexandru Csete.
+    Copyright (C)  2001-2007  Alexandru Csete.
 
-    Authors: Alexandru Csete <csete@users.sourceforge.net>
+    Authors: Alexandru Csete <oz9aec@gmail.com>
 
     Comments, questions and bugreports should be submitted via
     http://sourceforge.net/projects/groundstation/
@@ -91,8 +91,8 @@ typedef struct {
 	GdkGC            *gc2;             /*!< Graphics context (inverted). */
 	guint             width;           /*!< Canvas width. */
 	guint             height;          /*!< Canvas height. */
-	lcd_coor_t        digits[12];      /*!< Starting points for all digits. */
-	lcd_coor_t        others[5];       /*!< Starting points for other elements. */
+	lcd_coor_t        digits[13];      /*!< Starting points for all digits. */
+	lcd_coor_t        dots[2];       /*!< Starting points for dots. */
 	guint             dlw;             /*!< Width of large digits. */
 	guint             dlh;             /*!< Height of large digits. */
 	guint             clw;             /*!< Width of large separator. */
@@ -102,11 +102,14 @@ typedef struct {
 	GdkColor          bg;              /*!< Background color. */
 	GdkColor          fg;              /*!< Foreground color. */
 	gboolean          exposed;         /*!< Exposed flag. */
+	gboolean	  manual;	   /*!< Manual freq entry flag. */
+	gint		  digit;	   /*!< Current digit when in manual entry. */
 
 	vfo_t             vfo;             /*!< Current VFO. */
 	gdouble           freq1;           /*!< Main frequency value. */
 	gdouble           freq2;           /*!< Secondary frequency value. */
-	gchar             freqs1[9];       /*!< Frequency array. */
+	gdouble           freqm;           /*!< Manually entered frequency value. */
+	gchar             freqs1[10];      /*!< Frequency array. */
 	gint              rit;             /*!< Current RIT value. */
 	gchar             rits[4];         /*!< -9999 Hz but last digit not shown */
 	gint              xit;             /*!< Current XIT value. */
@@ -115,6 +118,10 @@ typedef struct {
 
 
 GtkWidget *rig_gui_lcd_create (void);
-
+void       rig_gui_lcd_set_freq_digits  (freq_t freq);
+void       rig_gui_lcd_set_rit_digits   (shortfreq_t freq);
+void	   rig_gui_lcd_begin_manual_entry  (void);
+void	   rig_gui_lcd_clear_manual_entry  (void);
+void	   rig_gui_lcd_set_next_digit  (char digit);
 
 #endif
