@@ -1,10 +1,10 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
     Grig:  Gtk+ user interface for the Hamradio Control Libraries.
 
-    Copyright (C)  2001-2006  Alexandru Csete.
+    Copyright (C)  2001-2007  Alexandru Csete.
 
-    Authors: Alexandru Csete <csete@users.sourceforge.net>
+    Authors: Alexandru Csete <oz9aec@gmail.com>
 
     Comments, questions and bugreports should be submitted via
     http://sourceforge.net/projects/groundstation/
@@ -147,8 +147,11 @@ rig_gui_message_window_init  ()
 
 
 		hbox = gtk_hbox_new (FALSE, 10);
-		gtk_box_pack_start_defaults (GTK_BOX (hbox),
-					     create_message_list ());
+		gtk_box_pack_start (GTK_BOX (hbox),
+					     create_message_list (),
+					     TRUE,
+					     TRUE,
+					     0);
 
 		gtk_box_pack_start (GTK_BOX (hbox),
 				    create_message_summary (),
@@ -253,8 +256,6 @@ add_debug_message (const gchar *datetime,
 {
 	guint        total;     /* totalt number of messages */
 	gchar       *str;       /* string to show message count */
-	guint        numlines;  /* the number of lines in the message */
-	guint        i;
 	GtkTreeIter  item;      /* new item added to the list store */
 
 
@@ -262,7 +263,7 @@ add_debug_message (const gchar *datetime,
 	gtk_list_store_set (GTK_LIST_STORE (model), &item,
 			    MSG_LIST_COL_TIME, datetime,
 			    MSG_LIST_COL_SOURCE, source,
-			    MSG_LIST_COL_LEVEL, DEBUG_STR[debug_level],
+			    MSG_LIST_COL_LEVEL, _(DEBUG_STR[debug_level]),
 			    MSG_LIST_COL_MSG, message,
 			    -1);
 
@@ -599,7 +600,7 @@ create_message_list    ()
 	for (i = 0; i < MSG_LIST_COL_NUMBER; i++) {
 
 		renderer = gtk_cell_renderer_text_new ();
-		column = gtk_tree_view_column_new_with_attributes (MSG_LIST_COL_TITLE[i],
+		column = gtk_tree_view_column_new_with_attributes (_(MSG_LIST_COL_TITLE[i]),
 								   renderer,
 								   "text", i,
 								   NULL);
